"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.enable = exports.setDOMStorageItem = exports.removeDOMStorageItem = exports.getDOMStorageItems = exports.clear = void 0;
var safeStorage_1 = __importDefault(require("licia/safeStorage"));
var each_1 = __importDefault(require("licia/each"));
var isStr_1 = __importDefault(require("licia/isStr"));
var once_1 = __importDefault(require("licia/once"));
var jsonClone_1 = __importDefault(require("licia/jsonClone"));
var connector_1 = __importDefault(require("../lib/connector"));
var detectBrowser_1 = __importDefault(require("licia/detectBrowser"));
var localStore = safeStorage_1.default('local');
var sessionStore = safeStorage_1.default('session');
var browser = detectBrowser_1.default();
function clear(params) {
    var store = getStore(params.storageId);
    store.clear();
}
exports.clear = clear;
function getDOMStorageItems(params) {
    var store = getStore(params.storageId);
    var entries = [];
    each_1.default(jsonClone_1.default(store), function (val, key) {
        if (!isStr_1.default(val))
            return;
        entries.push([key, val]);
    });
    return {
        entries: entries,
    };
}
exports.getDOMStorageItems = getDOMStorageItems;
function removeDOMStorageItem(params) {
    var key = params.key, storageId = params.storageId;
    var store = getStore(storageId);
    store.removeItem(key);
}
exports.removeDOMStorageItem = removeDOMStorageItem;
function setDOMStorageItem(params) {
    var key = params.key, value = params.value, storageId = params.storageId;
    var store = getStore(storageId);
    store.setItem(key, value);
}
exports.setDOMStorageItem = setDOMStorageItem;
exports.enable = once_1.default(function () {
    if (browser.name === 'ie') {
        return;
    }
    each_1.default(['local', 'session'], function (type) {
        var store = type === 'local' ? localStore : sessionStore;
        var storageId = getStorageId(type);
        var originSetItem = store.setItem.bind(store);
        store.setItem = function (key, value) {
            if (!isStr_1.default(key) || !isStr_1.default(value))
                return;
            var oldValue = store.getItem(key);
            originSetItem(key, value);
            if (oldValue) {
                connector_1.default.trigger('DOMStorage.domStorageItemUpdated', {
                    key: key,
                    newValue: value,
                    oldValue: oldValue,
                    storageId: storageId,
                });
            }
            else {
                connector_1.default.trigger('DOMStorage.domStorageItemAdded', {
                    key: key,
                    newValue: value,
                    storageId: storageId,
                });
            }
        };
        var originRemoveItem = store.removeItem.bind(store);
        store.removeItem = function (key) {
            if (!isStr_1.default(key))
                return;
            var oldValue = store.getItem(key);
            if (oldValue) {
                originRemoveItem(key);
                connector_1.default.trigger('DOMStorage.domStorageItemRemoved', {
                    key: key,
                    storageId: storageId,
                });
            }
        };
        var originClear = store.clear.bind(store);
        store.clear = function () {
            originClear();
            connector_1.default.trigger('DOMStorage.domStorageItemsCleared', {
                storageId: storageId,
            });
        };
    });
});
function getStorageId(type) {
    return {
        securityOrigin: location.origin,
        isLocalStorage: type === 'local',
    };
}
function getStore(storageId) {
    var isLocalStorage = storageId.isLocalStorage;
    return isLocalStorage ? localStore : sessionStore;
}
